local L = LibStub("AceLocale-3.0"):GetLocale("Critline")
local core = Critline
local module


core.announce = core.template:CreateList("CritlineAnnounce")
module = core.announce
module:SetScript("OnShow", function(self)
	self.whisperTarget:SetAutoFocus(false)
	self.whisperTarget:Hide()
	self:ChannelInitialize()
	UIDropDownMenu_SetSelectedValue(self.channel, 14)
end)
module.button:SetText(CHAT_ANNOUNCE)
module.button:SetScript("OnClick", function()
	PlaySound("gsTitleOptionOK")
	module:Announce()
end)

module.selectedSpells = {}

module.name = CHAT_ANNOUNCE
module.parent = "Critline"
InterfaceOptions_AddCategory(module)

module.channel = CreateFrame("Frame", "CritlineAnnounceChannel", module, "UIDropDownMenuTemplate")
module.channel:SetPoint("TOPLEFT", module.tree, "BOTTOMLEFT")

module.whisperTarget = CreateFrame("EditBox", nil, module)
module.whisperTarget:SetWidth(144)
module.whisperTarget:SetHeight(32)
module.whisperTarget:SetPoint("LEFT", module.channel, "RIGHT", 0, 2)
module.whisperTarget:SetFontObject("ChatFontNormal")
module.whisperTarget:Hide()
module.whisperTarget:SetScript("OnEscapePressed", module.whisperTarget.Hide)
module.whisperTarget:SetScript("OnEnterPressed", function() module:Announce() end)

module.whisperTarget.left = module.whisperTarget:CreateTexture("BACKGROUND")
module.whisperTarget.left:SetTexture("Interface\\ChatFrame\\UI-ChatInputBorder-Left")
module.whisperTarget.left:SetWidth(128)
module.whisperTarget.left:SetHeight(32)
module.whisperTarget.left:SetPoint("LEFT")

module.whisperTarget.right = module.whisperTarget:CreateTexture("BACKGROUND")
module.whisperTarget.right:SetTexture("Interface\\ChatFrame\\UI-ChatInputBorder-Right")
module.whisperTarget.right:SetTexCoord(0.9375, 1, 0, 1)
module.whisperTarget.right:SetWidth(16)
module.whisperTarget.right:SetHeight(32)
module.whisperTarget.right:SetPoint("RIGHT")


function module:CreateChannel(curValue, curText)
	UIDropDownMenu_AddButton({
		text = curText,
		value = curValue,
		func = function()
			UIDropDownMenu_SetSelectedValue(self.channel, curValue)
			if curValue == 15 then
				self.whisperTarget:Show()
				self.whisperTarget:SetFocus()
			else
				self.whisperTarget:Hide()
			end
		end
	})
end


function module:Announce()
	local normal, crit, mychan
	local mychanID = UIDropDownMenu_GetSelectedValue(self.channel)
	local selectedTree = UIDropDownMenu_GetSelectedValue(self.tree)
	
	-- if mychanID == 0 or mychanID == nil then
		-- core:Message(CL_ANNOUNCE_MISSINGCHANNEL_TEXT..".")
		-- return
	-- end 
	
	if mychanID == 11 then
		mychan = "GUILD"
	elseif mychanID == 12 then
		mychan = "PARTY"
	elseif mychanID == 13 then
		mychan = "RAID"
	elseif mychanID == 14 then
		mychan = "SAY"
	elseif mychanID == 15 then
		mychan = "WHISPER"
		mychanID = self.whisperTarget:GetText()
		if mychanID == "" then
			core:Message(L["Invalid player name."])
			return
		end
	else
		mychan = "CHANNEL"
	end
	
	for spell in pairs(self.selectedSpells) do
		normal = CritlineDB[selectedTree][spell].normal and CritlineDB[selectedTree][spell].normal.amount
		crit = CritlineDB[selectedTree][spell].crit and CritlineDB[selectedTree][spell].crit.amount
		SendChatMessage(spell.." - "..L["Normal"]..": "..(normal or "n/a").." "..CL_CRIT_TEXT..": "..(crit or "n/a"), mychan, nil, mychanID)
	end
	
	wipe(self.selectedSpells)
	
	self:Update()
	self.whisperTarget:SetText("")
end


function module:LoadChannels()
	if IsInGuild() then
		module:CreateChannel(11, GUILD)
	end
	if GetNumPartyMembers() > 0 then
		module:CreateChannel(12, PARTY)
	end
	if GetNumRaidMembers() > 0 then
		module:CreateChannel(13, RAID)
	end
	
	module:CreateChannel(14, SAY)
	module:CreateChannel(15, WHISPER)

	local id, name
	
	for i = 1, 10 do
		id, name = select((i * 2 - 1), GetChannelList())
		if id then
			module:CreateChannel(id, name)
		end
	end
end


function module:ChannelInitialize()
	UIDropDownMenu_Initialize(self.channel, module.LoadChannels)
	UIDropDownMenu_SetWidth(self.channel, 120)
end