local L = LibStub("AceLocale-3.0"):GetLocale("Critline")
local core = Critline
local module


local function addTitle(self, text)
	self.title = self:CreateFontString(nil, nil, "GameFontNormalLarge")
	self.title:SetPoint("TOPLEFT", 16, -16)
	self.title:SetPoint("RIGHT", -16, 0)
	self.title:SetJustifyH("LEFT")
	self.title:SetJustifyV("TOP")
	self.title:SetText(text)
end


-- template function for options check buttons
local function createCheckButton(self)
	local checkButton = CreateFrame("CheckButton", nil, self, "OptionsBaseCheckButtonTemplate")
	checkButton:SetPushedTextOffset(0, 0)
	checkButton:SetScript("OnClick", function(self)
		if self:GetChecked() then
			PlaySound("igMainMenuOptionCheckBoxOn")
		else
			PlaySound("igMainMenuOptionCheckBoxOff")
		end
		CritlineSettingsCheckButton_OnClick(self)
	end)
	
	local checkButtonText = checkButton:CreateFontString(nil, nil, "GameFontHighlight")
	checkButtonText:SetPoint("LEFT", checkButton, "RIGHT", 0, 1)
	checkButton:SetFontString(checkButtonText)

	return checkButton
end


-- template function for options sliders
local function createSlider(self, name)
	local slider = CreateFrame("Slider", name, self, "OptionsSliderTemplate")
	_G[name.."Text"]:SetFontObject("GameFontNormal")
	-- font string for current value
	slider.valueText = slider:CreateFontString(nil, "BACKGROUND", "GameFontHighlightSmall")
	slider.valueText:SetPoint("CENTER", 0, -15)
	
	return slider
end


-- template function for options color picker buttons
local function createColorButton(self)
	local button = CreateFrame("Button", nil, self)
	button:SetWidth(26)
	button:SetHeight(26)
	button:SetPushedTextOffset(0, 0)
	
	button:SetNormalTexture("Interface\\ChatFrame\\ChatFrameColorSwatch")
	button.swatch = button:GetNormalTexture()
	button.swatch:SetTexCoord(0, 1, 0, 1)
	
	local buttonText = button:CreateFontString(nil, nil, "GameFontHighlight")
	buttonText:SetPoint("LEFT", button, "RIGHT")
	buttonText:SetJustifyH("LEFT")
	button:SetFontString(buttonText)
	
	button:SetScript("OnClick", function(self)
		CritlineSettings_ColorPicker(self:GetID())
	end)
	button:SetScript("OnEnter", function(self)
		if self.tooltipText then
			GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
			GameTooltip:SetText(self.tooltipText, nil, nil, nil, nil, 1)
		end
	end)
	button:SetScript("OnLeave", function()
		GameTooltip:Hide()
	end)
	
	return button
end


core.settings = CreateFrame("Frame")
module = core.settings
addTitle(module, "Critline")
module:RegisterEvent("ADDON_LOADED")
module:SetScript("OnEvent", function(self, event, addon)
	if addon == "Critline" then
		self:LoadSettings()
	end
end)
module.name = "Critline"
InterfaceOptions_AddCategory(module)

module.subText = module:CreateFontString(nil, nil, "GameFontHighlightSmall")
module.subText:SetPoint("TOPLEFT", module.title, "BOTTOMLEFT", 0, -8)
module.subText:SetPoint("RIGHT", -32, 0)
module.subText:SetHeight(32)
module.subText:SetJustifyH("LEFT")
module.subText:SetJustifyV("TOP")
module.subText:SetNonSpaceWrap(true)
module.subText:SetText(L["Saves your high normal and critical damage records and flashes a message if you break the record."])


SlashCmdList["CRITLINESETTINGS"] = function(msg)
	InterfaceOptionsFrame_OpenToCategory(CritlineSettingsBasic)
end

SLASH_CRITLINESETTINGS1 = "/critline"


module.basic = CreateFrame("Frame")
addTitle(module.basic, L["Basic options"])
module.basic.name = L["Basic options"]
module.basic.parent = "Critline"
InterfaceOptions_AddCategory(module.basic)

do	-- create basic options
	module.basic.options = {
		checkButtons = {},
	}
	
	local options = module.basic.options
	local option
	
	for i = 1, 11 do
		option = createCheckButton(module.basic)
		if i == 1 then
			option:SetPoint("TOPLEFT", module.basic.title, "BOTTOMLEFT", -2, -16)
		elseif i == 7 then
			option:SetPoint("TOPLEFT", module.basic.title, "BOTTOM", 0, -16)
		else
			option:SetPoint("TOP", options.checkButtons[i - 1], "BOTTOM", 0, -8)
		end
		options.checkButtons[i] = option
	end
	
	option = createSlider(module.basic, "CritlineLevelAdjust")
	option:SetMinMaxValues(-1, 10)
	option:SetValueStep(1)
	option:SetPoint("TOPLEFT", options.checkButtons[11], "BOTTOMLEFT", 4, -24)
	option:SetScript("OnValueChanged", function(self)
		local lvAdj = self:GetValue()
		if lvAdj == -1 then
			self.valueText:SetText(OFF)
		else
			self.valueText:SetText(tostring(lvAdj))
		end
		if CritlineSettings then
			CritlineSettings.levelAdjust = lvAdj
		end
	end)
	CritlineLevelAdjustLow:SetText(OFF)
	CritlineLevelAdjustHigh:SetText("10")
	CritlineLevelAdjustText:SetText(L["Level adjustment"])
	option.tooltipText = L["If your level minus the target's level is more than this setting, the record will not be saved."]
	options.slider = option
end


module.advanced = CreateFrame("Frame")
addTitle(module.advanced, L["Advanced options"])
module.advanced.name = L["Advanced options"]
module.advanced.parent = "Critline"
InterfaceOptions_AddCategory(module.advanced)

do	-- create advanced options
	module.advanced.options = {
		checkButtons = {},
		colorButtons = {},
		buttons = {},
		sliders = {},
	}
	
	local options = module.advanced.options
	local option
	
	for i = 1, 9 do
		option = createCheckButton(module.advanced)
		if i == 1 then
			option:SetPoint("TOPLEFT", module.advanced.title, "BOTTOMLEFT", -2, -16)
		elseif i == 5 then
			option:SetPoint("TOPLEFT", module.advanced.title, "BOTTOM", 0, -16)
		else
			option:SetPoint("TOP", options.checkButtons[i - 1], "BOTTOM", 0, -8)
		end
		options.checkButtons[i] = option
	end
	
	-- splash frame spell name color
	option = createColorButton(module.advanced)
	option:SetID(1)
	option:SetPoint("TOPLEFT", options.checkButtons[4], "BOTTOMLEFT", 0, -8)
	options.colorButtons[1] = option
	
	-- splash frame amount color
	option = createColorButton(module.advanced)
	option:SetID(2)
	option:SetPoint("TOPLEFT", options.colorButtons[1], "BOTTOMLEFT", 0, -8)
	options.colorButtons[2] = option
	
	-- move splash screen
	option = CreateFrame("Button", "CritlineMoveSplashButton", module.advanced, "UIPanelButtonTemplate2")
	option:SetPoint("TOPLEFT", options.colorButtons[2], "BOTTOMLEFT", 4, -16)
	option:SetWidth(148)
	option:SetHeight(24)
	option:SetScript("OnClick", function()
		InterfaceOptionsFrame:Hide()
		core.splash.locked = false
		core.splash:EnableMouse(true)
	end)
	options.buttons[1] = option
	
	-- edit tooltip format
	option = CreateFrame("Button", "CritlineEditTooltipButton", module.advanced, "UIPanelButtonTemplate2")
	option:SetPoint("TOP", options.buttons[1], "BOTTOM", 0, -8)
	option:SetWidth(148)
	option:SetHeight(24)
	option:SetScript("OnClick", function()
		StaticPopup_Show("CRITLINE_EDIT_TOOLTIP_FORMAT")
	end)
	options.buttons[2] = option
	
	-- summary sort dropdown
	option = CreateFrame("Frame", "CritlineSummarySorting", module.advanced, "UIDropDownMenuTemplate")
	option:SetPoint("TOPLEFT", options.buttons[2], "BOTTOMLEFT", -13, -24)
	option.label = option:CreateFontString(nil, "BACKGROUND", "GameFontNormalSmall")
	option.label:SetPoint("BOTTOMLEFT", option, "TOPLEFT", 16, 3)
	option.label:SetText(L["Sort summary spells:"])
	options.summarySort = option
	
	local sliderName = "CritlineAdvancedSlider"
	
	local sliderData = {
		{
			minValue = 0.5,
			maxValue = 1.5,
			defaultValue = 1,
			valueStep = 0.05,
			relativeTo = options.checkButtons[9],
			text = L["Summary frame scale"],
			tooltipText = L["Sets the scale of the summary frame."],
			lowText = "50 %",
			highText = "150%",
			onValueChanged = function(self)
				local value = self:GetValue()
				core.display:SetScale(value)
				self.valueText:SetText(format("%.0f%%", value * 100))
				CritlineSettings.mainScale = value
			end,
		},
		{
			minValue = 0.5,
			maxValue = 1,
			defaultValue = 1,
			valueStep = 0.05,
			relativeTo = sliderName.."1",
			text = L["Splash frame scale"],
			tooltipText = L["Sets the scale of the splash frame."],
			lowText = "50%",
			highText = "100%",
			onValueChanged = function(self)
				local value = self:GetValue()
				core.splash:SetScale(value)
				self.valueText:SetText(format("%.0f%%", value * 100))
				CritlineSettings.splashScale = value
			end,
		},
		{
			minValue = 0.5,
			maxValue = 3,
			defaultValue = 2,
			valueStep = 0.5,
			relativeTo = sliderName.."2",
			text = L["Splash frame timer"],
			tooltipText = L["Sets the number of seconds you wish to display the splash frame."],
			lowText = "0.5",
			highText = "3",
			onValueChanged = function(self)
				local value = self:GetValue()
				self.valueText:SetText(tostring(value))
				CritlineSettings.splashTimer = value
				core.splash:SetTimeVisible(value)
			end,
		},
	}
	
	local x, y, anchor, relativePoint = 4, -24, "TOPLEFT", "BOTTOMLEFT"
	
	for i, v in ipairs(sliderData) do
		option = createSlider(module.advanced, sliderName..i)
		if i == 2 then
			x, y, anchor, relativePoint = 0, -40, "TOP", "BOTTOM"
		end
		option:SetMinMaxValues(v.minValue, v.maxValue)
		option:SetValueStep(v.valueStep)
		option:SetPoint(anchor, v.relativeTo, relativePoint, x, y)
		option:SetScript("OnValueChanged", v.onValueChanged)
		_G[sliderName..i.."Low"]:SetText(v.lowText)
		_G[sliderName..i.."High"]:SetText(v.highText)
		_G[sliderName..i.."Text"]:SetText(v.text)
		option.tooltipText = v.tooltipText
		options.sliders[i] = option
	end
end


-- "edit tooltip format" popup
StaticPopupDialogs["CRITLINE_EDIT_TOOLTIP_FORMAT"] = {
	text =
		"$sn - Spell name\n\n"..

		"$na - Amount for normal record\n"..
		"$nt - Target name for normal record\n"..
		"$nl - Target level for normal record\n\n"..

		"$ca - Amount for crit record\n"..
		"$ct - Target name for crit record\n"..
		"$cl - Target level for crit record\n\n"..
		
		"\\n - New line\n"..
		"\\t - Right aligned\n\n"..
		
		"Leave empty to set to default."
	,
	button1 = ACCEPT,
	button2 = CANCEL,
	hasEditBox = 1,
	OnAccept = function(self)
		local text = self.editBox:GetText()
		CritlineSettings.tooltipFormat = text
		core:RebuildAllTooltips()
	end,
	EditBoxOnEnterPressed = function(self)
		CritlineSettings.tooltipFormat = self:GetText()
		core:RebuildAllTooltips()
		self:GetParent():Hide()
	end,
	EditBoxOnEscapePressed = function(self)
		self:GetParent():Hide()
	end,
	OnShow = function(self)
		self.editBox:SetText(CritlineSettings.tooltipFormat)
		self.editBox:SetFocus()
	end,
	OnHide = function(self)
		if ChatFrameEditBox:IsVisible() then
			ChatFrameEditBox:SetFocus()
		end
		self.editBox:SetText("")
	end,
	timeout = 0,
	exclusive = 0,
	whileDead = 1,
}


function module:LoadSettings()
	local options, button
	local basicOptions = {
		{
			text = L["Record damage"],
			tooltipText = L["Check to enable damage events to be recorded."],
			setting = "dmgRecord",
		},
		{
			text = L["Record healing"],
			tooltipText = L["Check to enable healing events to be recorded."],
			setting = "healRecord",
		},
		{
			text = L["Record pet damage"],
			tooltipText = L["Check to enable pet damage events to be recorded."],
			setting = "petRecord",
		},
		{
			text = L["Show damage"],
			tooltipText = L["Show damage in summary frame."],
			setting = "dmgDisplay",
		},
		{
			text = L["Show healing"],
			tooltipText = L["Show healing in summary frame."],
			setting = "healDisplay",
		},
		{
			text = L["Show pet damage"],
			tooltipText = L["Show pet damage in summary frame."],
			setting = "petDisplay",
		},
		{
			text = L["Minimap button"],
			tooltipText = L["Show button on minimap."],
			setting = "showMinimap",
		},
		{
			text = L["Splash frame"],
			tooltipText = L["Shows the new record on the middle of the screen."],
			setting = "showSplash",
		},
		{
			text = L["Play sound"],
			tooltipText = L["Plays a sound on a new record."],
			setting = "playSound",
		},
		{
			text = L["Screenshot"],
			tooltipText = L["Saves a screenshot on a new record."],
			setting = "screenshot",
		},
		{
			text = L["Detailed summary"],
			tooltipText = L["Use detailed format in the Critline summary tooltip."],
			setting = "detailedTooltip",
		},
	}
	
	options = module.basic.options
	
	for i, v in ipairs(basicOptions) do
		button = options.checkButtons[i]
		button:SetText(v.text)
		button.tooltipText = v.tooltipText
		button.setting = v.setting
		button:SetChecked(CritlineSettings[v.setting])
	end
	
	options.slider:SetValue(CritlineSettings.levelAdjust)
	
	local advancedOptions = {
		{
			text = L["Invert damage filter"],
			tooltipText = L["Enable to include rather than exclude selected spells in the damage filter."],
			setting = "dmgInvert",
		},
		{
			text = L["Invert healing filter"],
			tooltipText = L["Enable to include rather than exclude selected spells in the healing filter."],
			setting = "healInvert",
		},
		{
			text = L["Invert pet filter"],
			tooltipText = L["Enable to include rather than exclude selected spells in the pet filter."],
			setting = "petInvert",
		},
		{
			text = L["Use combat text splash"],
			tooltipText = L["Enable to use scrolling combat text instead of the default splash frame."],
			setting = "sctSplash",
		},
		{
			text = L["Record PvE"],
			tooltipText = L["Disable to ignore records where the target is an NPC."],
			setting = "recordPvE",
		},
		{
			text = L["Record PvP"],
			tooltipText = L["Disable to ignore records where the target is a player."],
			setting = "recordPvP",
		},
		{
			text = L["Suppress mind control"],
			tooltipText = L["Suppress all records while mind controlled."],
			setting = "suppressMC",
		},
		{
			text = L["Don't filter magic"],
			tooltipText = L["Enable to let magical damage ignore the level adjustment."],
			setting = "dontFilterMagic",
		},
		{
			text = L["Ignore self damage"],
			tooltipText = L["Enable to ignore self inflicted damage."],
			setting = "noSelfDamage",
		},
	}
	
	options = module.advanced.options
	
	for i, v in ipairs(advancedOptions) do
		button = options.checkButtons[i]
		button:SetText(v.text)
		button.tooltipText = v.tooltipText
		button.setting = v.setting
		button:SetChecked(CritlineSettings[v.setting])
	end
	
	options.buttons[1]:SetText(L["Move splash screen"])
	
	options.buttons[2]:SetText(L["Edit tooltip format"])

	UIDropDownMenu_Initialize(options.summarySort, function(self)
		local items = {
			{
				text = L["Alphabetically"],
				value = "alpha",
				func = function()
					UIDropDownMenu_SetSelectedValue(self, "alpha")
					CritlineSettings.summarySort = "alpha"
					core:RebuildAllTooltips()
				end
			},
			{
				text = L["By crit record"],
				value = "crit",
				func = function()
					UIDropDownMenu_SetSelectedValue(self, "crit")
					CritlineSettings.summarySort = "crit"
					core:RebuildAllTooltips()
				end
			},
			{
				text = L["By normal record"],
				value = "normal",
				func = function()
					UIDropDownMenu_SetSelectedValue(self, "normal")
					CritlineSettings.summarySort = "normal"
					core:RebuildAllTooltips()
				end
			},
		}
		for _, item in ipairs(items) do
			UIDropDownMenu_AddButton(item)
		end
	end)

	UIDropDownMenu_SetWidth(options.summarySort, 120)
	UIDropDownMenu_SetSelectedValue(options.summarySort, CritlineSettings.summarySort)

	options.colorButtons[1]:SetText(L["Spell color"])
	options.colorButtons[1].tooltipText = L["Set the color for the spell text in the splash frame."]
	options.colorButtons[1].swatch:SetVertexColor(CritlineSettings.spellColor.r, CritlineSettings.spellColor.g, CritlineSettings.spellColor.b)
	
	options.colorButtons[2]:SetText(L["Amount color"])
	options.colorButtons[2].tooltipText = L["Set the color for the amount text in the splash frame."]
	options.colorButtons[2].swatch:SetVertexColor(CritlineSettings.amountColor.r, CritlineSettings.amountColor.g, CritlineSettings.amountColor.b)
	
	options.sliders[1]:SetValue(CritlineSettings.mainScale)
	options.sliders[2]:SetValue(CritlineSettings.splashScale)
	options.sliders[3]:SetValue(CritlineSettings.splashTimer)
end


function CritlineSettingsCheckButton_OnClick(self)
	local checked = self:GetChecked()
	core:Debug(self.setting..(checked and " on" or " off"))
	CritlineSettings[self.setting] = checked and true or false
	if self.setting:find("Display$") then
		core.display:UpdateTree(self.setting:sub(1, -8), checked)
	elseif self.setting == "detailedTooltip" or self.setting:find("Invert$") then
		core:RebuildAllTooltips()
	elseif self.setting == "showMinimap" then
		if checked then
			core.minimap:Show()
		else
			core.minimap:Hide()
		end
	end
end


local CritlineSettings_CancelFunc = {
	[1] = function(x) CritlineSettings_CancelColor(1, x) end,
	[2] = function(x) CritlineSettings_CancelColor(2, x) end,
	[3] = function(x) CritlineSettings_CancelColor(3, x) end,
	[4] = function(x) CritlineSettings_CancelColor(4, x) end,
	[5] = function(x) CritlineSettings_CancelColor(5, x) end,
	[6] = function(x) CritlineSettings_CancelColor(6, x) end
}

local CritlineSettings_SetColorFunc = {
	[1] = function() CritlineSettings_SetColor(1) end,
	[2] = function() CritlineSettings_SetColor(2) end,
	[3] = function() CritlineSettings_SetColor(3) end,
	[4] = function() CritlineSettings_SetColor(4) end,
	[5] = function() CritlineSettings_SetColor(5) end,
	[6] = function() CritlineSettings_SetColor(6) end
}


function CritlineSettings_ColorPicker(id)
	CloseMenus()
	local texture = module.advanced.options.colorButtons[id].swatch
	if not texture.a then
		texture.a = 1
	end
	ColorPickerFrame.func = CritlineSettings_SetColorFunc[id]
	ColorPickerFrame:SetColorRGB(texture.r, texture.g, texture.b)
	ColorPickerFrame.previousValues = {r = texture.r, g = texture.g, b = texture.b, opacity =texture.a}
	ColorPickerFrame.cancelFunc = CritlineSettings_CancelFunc[id]
	ColorPickerFrame.hasOpacity = false
	
	InterfaceOptionsFrame:Hide()
	ColorPickerFrame:Show()
end


function CritlineSettings_SetColor(id)
	local r, g, b = ColorPickerFrame:GetColorRGB()
	local texture = module.advanced.options.colorButtons[id].swatch
	--SHTSetOpacity(id)
	texture:SetVertexColor(r, g, b)
	texture.r = r
	texture.g = g
	texture.b = b
	if id == 1 then
		CritlineSettings.spellColor.r = r
		CritlineSettings.spellColor.g = g
		CritlineSettings.spellColor.b = b
	elseif id == 2 then
		CritlineSettings.amountColor.r = r
		CritlineSettings.amountColor.g = g
		CritlineSettings.amountColor.b = b
	end
end


function CritlineSettings_CancelColor(id, prev)
	local texture = module.advanced.options.colorButtons[id].swatch
	local r = prev.r
	local g = prev.g
	local b = prev.b
	local a = prev.opacity
	texture:SetVertexColor(r, g, b)
	texture:SetAlpha( a )
	texture.r = r
	texture.g = g
	texture.b = b
	texture.a = a
end